<?php

namespace Lc\ShopBundle\Controller\Backend;

use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\EntityRepository;
use EasyCorp\Bundle\EasyAdminBundle\Controller\EasyAdminController;
use EasyCorp\Bundle\EasyAdminBundle\Event\EasyAdminEvents;
use FOS\UserBundle\Model\UserManagerInterface;
use Lc\ShopBundle\Context\FilterMultipleMerchantsInterface;
use Lc\ShopBundle\Context\MerchantInterface;
use Lc\ShopBundle\Context\MerchantUtilsInterface;
use Lc\ShopBundle\Context\OrderUtilsInterface;
use Lc\ShopBundle\Context\ReminderInterface;
use Lc\ShopBundle\Context\SeoInterface;
use Lc\ShopBundle\Context\StatusInterface;
use Lc\ShopBundle\Context\TreeInterface;
use Lc\ShopBundle\Form\Backend\Common\AbstractEditPositionType;
use Lc\ShopBundle\Services\Utils;
use Lc\ShopBundle\Services\UtilsManager;
use Mailjet\MailjetSwiftMailer\SwiftMailer\MailjetTransport;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\CollectionType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormEvent;
use Symfony\Component\Form\FormEvents;
use Symfony\Component\Security\Core\Security;
use Symfony\Contracts\Translation\TranslatorInterface;

class AdminController extends EasyAdminController
{
        protected $security;
        protected $userManager;
        protected $em;
        protected $utils;
        protected $merchantUtils;
        protected $mailjetTransport;
        protected $orderUtils;
        protected $translator;

        public function __construct(Security $security, UserManagerInterface $userManager, EntityManagerInterface $em,
                                    MailjetTransport $mailjetTransport, UtilsManager $utilsManager, TranslatorInterface $translator)
        {
                $this->security = $security;
                $this->userManager = $userManager;
                $this->em = $em;
                $this->mailjetTransport = $mailjetTransport;
                $this->utils = $utilsManager->getUtils();
                $this->merchantUtils = $utilsManager->getMerchantUtils();
                $this->orderUtils =  $utilsManager->getOrderUtils() ;;
                $this->translator = $translator;
        }

        public function createCustomForm($class, $action, $parameters, $data = true)
        {

                $options = array();
                if ($data) $options['data'] = $parameters['entity'];
                $options['action'] = $this->generateUrl('easyadmin', array(
                        'action' => $action,
                        'entity' => $this->entity['name'],
                        'id' => $parameters['entity']->getId()
                ));

                return $this->createForm($class, null, $options);
        }


        /**
         * Réécriture de show action pr rediriger vers l'édition
         */
/*        public function showAction()
        {
                $id = $this->request->query->get('id');
                $entity = $this->request->query->get('entity');

                return $this->redirectToRoute('easyadmin', [
                        'action' => 'edit',
                        'entity' => $entity,
                        'id' => $id
                ]);
        }*/


        /**
         * Réécriture de removeEntity pour passer les status à -1
         */
        protected function removeEntity($entity)
        {
                if ($entity instanceof StatusInterface) {
                        $entity->setStatus(-1);
                        $this->em->persist($entity);
                        $this->em->flush();

                } else {
                        parent::removeEntity($entity);
                }
        }

        protected function commonDqlFilterQueryBuilder($entityClass, $dqlFilter)
        {
                if ($pos = strpos($dqlFilter, 'currentMerchant')) {
                        $dqlFilter = sprintf(str_replace('currentMerchant', $this->getUser()->getMerchant()->getId(), $dqlFilter));
                }

                if (new $entityClass instanceof StatusInterface && strpos($dqlFilter, 'entity.status') === false) {
                        if ($dqlFilter) $dqlFilter .= sprintf(' AND entity.status > = 0');
                        else $dqlFilter .= sprintf(' entity.status > = 0');
                }

                return $dqlFilter;
        }

        protected function commonQueryFilter($entityClass, $queryBuilder)
        {
                if (new $entityClass instanceof FilterMultipleMerchantsInterface) {
                        $queryBuilder->andWhere(':currentMerchant MEMBER OF entity.merchants')
                                ->setParameter(':currentMerchant', $this->getUser()->getMerchant()->getId());
                }

        }

        protected function createSearchQueryBuilder($entityClass, $searchQuery, array $searchableFields, $sortField = null, $sortDirection = null, $dqlFilter = null)
        {
                $dqlFilter = $this->commonDqlFilterQueryBuilder($entityClass, $dqlFilter);
                $queryBuilder = parent::createSearchQueryBuilder($entityClass, $searchQuery, $searchableFields, $sortField, $sortDirection, $dqlFilter);
                $this->commonQueryFilter($entityClass, $queryBuilder);
                return $queryBuilder;
        }

        protected function createListQueryBuilder($entityClass, $sortDirection, $sortField = null, $dqlFilter = null)
        {
                $dqlFilter = $this->commonDqlFilterQueryBuilder($entityClass, $dqlFilter);
                $queryBuilder = parent::createListQueryBuilder($entityClass, $sortDirection, $sortField, $dqlFilter);
                $this->commonQueryFilter($entityClass, $queryBuilder);
                return $queryBuilder;
        }


        public function renderTemplate($actionName, $templatePath, array $parameters = [])
        {

                $reminderRepo = $this->em->getRepository(ReminderInterface::class);
                $easyadmin = $this->request->attributes->get('easyadmin');
                $entityName = $easyadmin['entity']['name'];
                $id=null;
                if($easyadmin['item'])$id = $easyadmin['item']->getId();

                $reminders = array('reminders' => $reminderRepo->findByEasyAdminConfig($actionName, $entityName, $id ));

                $parameters = array_merge(
                        $parameters,
                        $this->getTwigExtraParameters(),
                        $reminders
                );

                return parent::renderTemplate($actionName, $templatePath, $parameters);
        }

        public function getTwigExtraParameters()
        {
                $repositoryMerchant = $this->getDoctrine()->getRepository(MerchantInterface::class);
                $merchants = $repositoryMerchant->findAll();

                $user = $this->security->getUser();

                return [
                        'merchants' => $merchants,
                        'current_merchant' => ($user && $user->getMerchant()) ? $user->getMerchant() : null,
                ];
        }


        public function listChildrenAction()
        {
                $this->dispatch(EasyAdminEvents::PRE_LIST);

                $id = $this->request->query->get('id');
                $this->entity['list']['dql_filter'] = "entity.parent = " . $id;

                $easyadmin = $this->request->attributes->get('easyadmin');
                $entity = $easyadmin['item'];

                $fields = $this->entity['list']['fields'];
                $paginator = $this->findAll($this->entity['class'], $this->request->query->get('page', 1), $this->entity['list']['max_results'], $this->request->query->get('sortField'), $this->request->query->get('sortDirection'), $this->entity['list']['dql_filter']);

                $this->dispatch(EasyAdminEvents::POST_LIST, ['paginator' => $paginator]);

                $parameters = [
                        'paginator' => $paginator,
                        'fields' => $fields,
                        'batch_form' => $this->createBatchForm($this->entity['name'])->createView(),
                        'delete_form_template' => $this->createDeleteForm($this->entity['name'], '__id__')->createView(),
                        'entity' => $entity,
                ];

                return $this->executeDynamicMethod('render<EntityName>Template', ['list', $this->entity['templates']['list'], $parameters]);
        }


        public function sortAction()
        {

                $this->dispatch(EasyAdminEvents::PRE_LIST);
                $entity = null;

                //Replace this with query builder function, do not use finAll of easyAdmin
                if ($this->request->query->get('id')) {
                        $id = $this->request->query->get('id');
                        $this->entity['list']['dql_filter'] = "entity.parent = " . $id;

                        $easyadmin = $this->request->attributes->get('easyadmin');
                        $entity = $easyadmin['item'];
                }

                if ($this->entity['list']['dql_filter']) $this->entity['list']['dql_filter'] .= sprintf(' AND entity.status = 1');
                else $this->entity['list']['dql_filter'] .= sprintf(' entity.status = 1');

                $fields = $this->entity['list']['fields'];
                $paginator = $this->findAll($this->entity['class'], $this->request->query->get('page', 1), 500, $this->request->query->get('sortField'), $this->request->query->get('sortDirection'), $this->entity['list']['dql_filter']);

                $this->dispatch(EasyAdminEvents::POST_LIST, ['paginator' => $paginator]);

                $positionForm = $this->createFormBuilder(array('entities', $paginator->getCurrentPageResults()))
                        ->add('entities', CollectionType::class, array(
                                'required' => true,
                                'allow_add' => true,
                                'entry_type' => AbstractEditPositionType::class,
                        ))
                        ->getForm();

                $positionForm->handleRequest($this->request);

                if ($positionForm->isSubmitted() && $positionForm->isValid()) {
                        $class = $this->entity['class'];
                        $repo = $this->em->getRepository($class);

                        $latsPos = 0;
                        foreach ($positionForm->get('entities')->getData() as $elm) {
                                $this->dispatch(EasyAdminEvents::PRE_UPDATE, ['entity' => $entity]);
                                $entity = $repo->find($elm['id']);
                                $entity->setPosition($elm['position']);
                                $this->em->persist($entity);
                                $this->dispatch(EasyAdminEvents::POST_UPDATE, ['entity' => $entity]);
                                $latsPos = $elm['position'];

                        }
                        //die();
                        //to do récupérer les élements hors ligne et incrémenter position
                        /*foreach ($repo->findBy(array('status'=> false)) as $offlineEntity) {
                                $latsPos++;
                                $offlineEntity->setPosition($latsPos);
                                $this->em->persist($offlineEntity);
                        }*/
                        $this->em->flush();

                        $this->addFlash('success', 'Position modifié', array(), 'mweb');

                        return $this->redirectToReferrer();

                }

                $parameters = [
                        'paginator' => $paginator,
                        'fields' => $fields,
                        'batch_form' => $this->createBatchForm($this->entity['name'])->createView(),
                        'delete_form_template' => $this->createDeleteForm($this->entity['name'], '__id__')->createView(),
                        'postion_form' => $positionForm->createView(),
                        'entity' => $entity,
                        'sortable' => true
                ];
                return $this->executeDynamicMethod('render<EntityName>Template', ['sortable', "@LcShop/backend/default/sortable.html.twig", $parameters]);
        }

        public function createEntityFormBuilder($entity, $view, $override = true)
        {

                $formBuilder = parent::createEntityFormBuilder($entity, $view);

                if ($override) $formBuilder = $this->overrideFormBuilder($formBuilder, $entity, $view);

                return $formBuilder;
        }

        public function overrideFormBuilder($formBuilder, $entity, $view)
        {
                $id = (null !== $entity->getId()) ? $entity->getId() : 0;

                if ($entity instanceof SeoInterface) {
                        $formBuilder->add('metaTitle', TextType::class, array(
                                'required' => false,
                                'translation_domain' => 'lcshop'
                        )
                        );
                        $formBuilder->add('metaDescription', TextareaType::class, array(
                                        'required' => false,
                                        'translation_domain' => 'lcshop'
                                )
                        );
                        /*$formBuilder->add('oldUrls', TextareaType::class, array(
                                        'required' => false,
                                        'translation_domain' => 'lcshop'
                                )
                        );*/
                }


                if ($entity instanceof StatusInterface) {
                        $formBuilder->add('status', ChoiceType::class, array(
                                        'choices' => array(
                                                'field.default.statusOptions.offline' => '0',
                                                'field.default.statusOptions.online' => '1'
                                        ),
                                        'expanded' => true,
                                        'required' => true,
                                        'translation_domain' => 'lcshop'
                                )
                        );
                }

                if ($entity instanceof TreeInterface) {

                        $formBuilder->add('parent', EntityType::class, array(
                                        'class' => $this->entity['class'],
                                        'query_builder' => function (EntityRepository $repo) use ($id) {
                                                return $repo->createQueryBuilder('e')
                                                        ->where('e.parent is NULL')
                                                        ->andWhere('e.status >= 0')
                                                        ->orderBy('e.status', "DESC");
                                        },
                                        'required' => false,
                                )
                        );
                }

                $formBuilder->addEventListener(FormEvents::POST_SET_DATA, function (FormEvent $event) {
                        $form = $event->getForm();
                        $childrens = $form->all();
                        $this->loopFormField($form, $childrens);
                });
                return $formBuilder;
        }

        private function loopFormField($form, $childrens)
        {
                foreach ($childrens as $child) {
                        $statusInterface = false;
                        $treeInterface = false;
                        $type = $child->getConfig()->getType()->getInnerType();;
                        if ($type instanceof EntityType) {
                                $passedOptions = $child->getConfig()->getOptions();
                                $classImplements = class_implements($passedOptions['class']);

                                $isFilterMerchantInterface = in_array('Lc\ShopBundle\Context\FilterMerchantInterface', $classImplements);
                                $isFilterMultipleMerchantsInterface = in_array('Lc\ShopBundle\Context\FilterMultipleMerchantsInterface', $classImplements);

                                if ($isFilterMerchantInterface || $isFilterMultipleMerchantsInterface) {

                                        if (in_array('Lc\ShopBundle\Context\StatusInterface', $classImplements)) {
                                                $statusInterface = true;
                                        }

                                        if (in_array('Lc\ShopBundle\Context\TreeInterface', $classImplements)) {
                                                $treeInterface = true;
                                        }
                                        $propertyMerchant = 'merchant';
                                        if ($isFilterMultipleMerchantsInterface) {
                                                $propertyMerchant .= 's';
                                        }
                                        $form->add($child->getName(), EntityType::class, array(
                                                        'class' => $this->em->getClassMetadata($passedOptions['class'])->getName(),
                                                        'label' => $passedOptions['label'],
                                                        'multiple' => isset($passedOptions['multiple']) ? $passedOptions['multiple'] : false,
                                                        'placeholder' => '--',
                                                        'translation_domain' => 'lcshop',
                                                        'query_builder' => function (EntityRepository $repo) use ($passedOptions, $propertyMerchant, $statusInterface, $treeInterface, $child) {
                                                                $queryBuilder = $repo->createQueryBuilder('e');
                                                                $propertyMerchant = 'e.' . $propertyMerchant;

                                                                if ($passedOptions['class'] == 'App\Entity\PointSale') {
                                                                        $queryBuilder->where(':currentMerchant MEMBER OF ' . $propertyMerchant);
                                                                } else {
                                                                        $queryBuilder->where($propertyMerchant . ' = :currentMerchant');
                                                                }

                                                                if ($statusInterface) {
                                                                        $queryBuilder->andWhere('e.status >= 0');
                                                                }
                                                                if ($treeInterface && $child->getName() == 'parent') {
                                                                        $queryBuilder->andWhere('e.parent is null');
                                                                }
                                                                $queryBuilder->setParameter(':currentMerchant', $this->getUser()->getMerchant()->getId());

                                                                return $queryBuilder;
                                                        },
                                                        'choice_label' => function ($choice) {
                                                                if ($choice instanceof StatusInterface && $choice->getStatus() == 0) {
                                                                        return $choice . ' [hors ligne]';
                                                                }
                                                                return $choice;

                                                        },
                                                        'required' => $passedOptions['required'],
                                                        'choice_attr' => $passedOptions['choice_attr'],
                                                )
                                        );
                                }
                        } else {
                                $subChildrens = $child->all();
                                if (count($subChildrens)) {
                                        $this->loopFormField($child, $subChildrens);
                                }
                        }
                }
        }
}

