<?php

namespace Lc\ShopBundle\Repository;

use App\Entity\DeliveryAvailabilityPointSale;
use App\Entity\DeliveryAvailabilityZone;
use Doctrine\ORM\QueryBuilder;
use Lc\ShopBundle\Context\DefaultRepositoryInterface;
use Lc\ShopBundle\Context\OrderShopInterface;
use Lc\ShopBundle\Model\OrderStatus;

/**
 * @method OrderShopInterface|null find($id, $lockMode = null, $lockVersion = null)
 * @method OrderShopInterface|null findOneBy(array $criteria, array $orderBy = null)
 * @method OrderShopInterface[]    findAll()
 * @method OrderShopInterface[]    findBy(array $criteria, array $orderBy = null, $limit = null, $offset = null)
 */
class OrderShopRepository extends BaseRepository implements DefaultRepositoryInterface
{

        public function getInterfaceClass()
        {
                return OrderShopInterface::class;
        }

        public function countValidOrderWithReductionCredit($reductionCredit, $user)
        {
                $query = $this->findByMerchantQuery();
                $query = $this->filterOrderValid($query);
                $query->select('count(e.id)');
                $query->andWhere('e.user = :user');
                $query->innerJoin('e.orderReductionCredits', 'orc');
                $query->andWhere('orc.reductionCredit = :reductionCredit');
                $query->setParameter('reductionCredit', $reductionCredit);
                $query->setParameter('user', $user);
                return $query->getQuery()->getSingleScalarResult();
        }


        public function countValidOrderWithReductionCart($reductionCart)
        {
                $query = $this->findByMerchantQuery();
                $query = $this->filterOrderValid($query);
                $query->select('count(e.id)');
                $query->join('e.orderReductionCarts', 'orc');
                $query->andWhere('orc.reductionCart = :reductionCart');
                $query->setParameter('reductionCart', $reductionCart);
                return $query->getQuery()->getSingleScalarResult();
        }


        public function countValidOrderWithReductionCartPerUser($reductionCart, $user)
        {
                $query = $this->findByMerchantQuery();
                $query = $this->filterOrderValid($query);
                $query->select('count(e.id)');
                $query->andWhere('e.user = :user');
                $query->join('e.orderReductionCarts', 'orc');
                $query->andWhere('orc.reductionCart = :reductionCart');
                $query->setParameter('reductionCart', $reductionCart);
                $query->setParameter('user', $user);
                return $query->getQuery()->getSingleScalarResult();
        }


        public function filterOrderValid(?QueryBuilder $query): QueryBuilder
        {
                $query->leftJoin('e.orderStatus', 'os');
                $query->andWhere('os.alias IN (:alias)');
                $query->setParameter('alias', OrderStatus::$statusAliasAsValid);

                return $query;
        }

        public function filterOrderCart($query)
        {
                $query->leftJoin('e.orderStatus', 'os');
                $query->andWhere('os.alias IN (:alias)');
                $query->setParameter('alias', OrderStatus::$statusAliasAsCart);

                return $query;
        }

        public function filterIsOffCircuit($query, $isOffCircuit)
        {
                if ($isOffCircuit) {
                        $query->andWhere('e.isOffCircuit = 1');
                } else {
                        $query->andWhere('e.isOffCircuit IS NULL OR e.isOffCircuit = 0');
                }

                return $query;
        }

        public function findCartCurrent($params)
        {
                $query = $this->findByMerchantQuery();


                if (isset($params['user'])) {
                        $query->andWhere('e.user = :user')->setParameter('user', $params['user']);
                }

                if (isset($params['visitor'])) {
                        $query->andWhere('e.visitor = :visitor')->setParameter('visitor', $params['visitor']);
                }

                $query = $this->filterOrderCart($query);

                $query->leftJoin('e.orderReductionCarts', 'orderReductionCarts')
                        ->addSelect('orderReductionCarts');

                $results = $query->getQuery()->getResult();

                if ($results) {
                        return $results[0];
                }

                return null;
        }

        public function findAllBy($params = [])
        {
                $query = $this->findByMerchantQuery();

                if (isset($params['count']) && $params['count']) {
                        $query->select('count(e.id)');
                }

                if (isset($params['dateStart']) || isset($params['dateEnd'])) {
                        $params['dateField'] = isset($params['dateField']) ? $params['dateField'] : 'validationDate';
                }

                if (isset($params['dateStart'])) {
                        $query->andWhere('e.' . $params['dateField'] . ' >= :dateStart')->setParameter('dateStart', $params['dateStart']);
                }
                if (isset($params['dateEnd'])) {
                        $query->andWhere('e.' . $params['dateField'] . ' <= :dateEnd')->setParameter('dateEnd', $params['dateEnd']);
                }

                if (isset($params['weekNumber'])) {
                        $query->andWhere('e.weekNumber = :weekNumber')->setParameter('weekNumber', $params['weekNumber']);
                }

                if (isset($params['isCart'])) {
                        $query = $this->filterOrderCart($query);
                }

                if (isset($params['isValid'])) {
                        $query = $this->filterOrderValid($query);
                }

                if (isset($params['user'])) {
                        $query->andWhere('e.user = :user')->setParameter('user', $params['user']);
                }

                if (isset($params['mergeComplementaryOrderShops'])) {
                        $query->andWhere('e.mainOrderShop IS NULL');
                        $query->leftJoin('e.complementaryOrderShops', 'complementaryOrderShops');

                        $query->leftJoin('complementaryOrderShops.orderStatus', 'complementaryOrderStatus');
                        $query->andWhere('complementaryOrderShops IS NULL OR complementaryOrderStatus.alias IN (:statusAliasValid)');
                        $query->setParameter('statusAliasValid', OrderStatus::$statusAliasAsValid);
                }

                if (isset($params['excludeComplementaryOrderShops'])) {
                        $query->andWhere('e.mainOrderShop IS NULL');
                }

                if (isset($params['isOffCircuit'])) {
                        $query = $this->filterIsOffCircuit($query, true);
                }

                if (isset($params['isCircuit'])) {
                        $query = $this->filterIsOffCircuit($query, false);
                        $query->leftJoin('e.deliveryPointSale', 'pointSale');
                        $query->andWhere('e.deliveryPointSale IS NULL OR pointSale.isDepository = 0');
                }

                if (isset($params['isDepository'])) {
                        $query = $this->filterIsOffCircuit($query, false);
                        $query->innerJoin('e.deliveryPointSale', 'pointSale');
                        $query->andWhere('pointSale.isDepository = 1');
                }

                if (isset($params['deliveryAvailability'])) {
                        $deliveryAvailability = $params['deliveryAvailability'] ;

                        $deliveryAvailabilityZone = ($deliveryAvailability instanceof DeliveryAvailabilityZone) ? $deliveryAvailability : false;
                        $deliveryAvailabilityPointSale = ($deliveryAvailability instanceof DeliveryAvailabilityPointSale) ? $deliveryAvailability : false;

                        if($deliveryAvailabilityZone){
                                $query->andWhere('e.deliveryAvailabilityZone = :deliveryAvailabilityZone');
                                $query->setParameter('deliveryAvailabilityZone', $deliveryAvailabilityZone);
                        }

                        if($deliveryAvailabilityPointSale) {
                                $query->andWhere('e.deliveryAvailabilityPointSale = :deliveryAvailabilityPointSale');
                                $query->setParameter('deliveryAvailabilityPointSale', $deliveryAvailabilityPointSale);
                        }
                }
                else{
                        $query->leftJoin('e.deliveryAvailabilityZone', 'deliveryAvailabilityZone');
                        $query->leftJoin('deliveryAvailabilityZone.deliverySlot', 'deliverySlotZone');

                        $query->leftJoin('e.deliveryAvailabilityPointSale', 'deliveryAvailabilityPointSale');
                        $query->leftJoin('deliveryAvailabilityPointSale.deliverySlot', 'deliverySlotPointSale');
                }

                if (isset($params['orderBy'])) {
                        $query->orderBy('e.' . $params['orderBy'], isset($params['orderByDirection']) ? $params['orderByDirection'] : 'DESC');
                } else {
                        $query->orderBy('e.id', 'DESC');
                }


                if (isset($params['count']) && $params['count']) {
                        return $query->getQuery()->getSingleScalarResult();
                } else {
                        return $query->getQuery()->getResult();
                }
        }

        public function findLastOrderValidOfWeek($weekNumber)
        {
                $query = $this->findByMerchantQuery();
                $query = $this->filterOrderValid($query);
                $query->andWhere('e.weekNumber = :weekNumber');
                $query->setParameter('weekNumber', $weekNumber);
                $query->orderBy('e.validationDate', 'DESC');
                $query->setMaxResults(1);

                return $query->getQuery()->getOneOrNullResult();

        }

        public function countValidOrderProductsOfWeekByProduct($weekNumber, $productId)
        {
                $query = $this->findByMerchantQuery();
                $query = $this->filterOrderValid($query);
                $query->andWhere('e.weekNumber = :weekNumber');
                $query->setParameter('weekNumber', $weekNumber);
                $query->leftJoin('e.orderProducts', 'orderProduct');
                $query->andWhere('orderProduct.product = :product');
                $query->setParameter('product', $productId);
                $query->select('SUM(orderProduct.quantityOrder) as quantity');

                return $query->getQuery()->getOneOrNullResult();

        }
}
